;------------------- Listing 4 - tsrasm.asm ------------------------
         dosseg
         .model small
         .data
         public        _running
         extrn _myss       : word
         extrn _stack      : word
         extrn _suspending : word
         extrn _olddisk    : dword
         extrn _oldvideo   : dword
         extrn _diskflag   : word
         extrn _videoflag  : word
intss    dw    ?
intsp    dw    ?
_running db    0
         .code
extrn    _popup_tsr : near
public   _getsp
; -------- return the stack pointer
;        stackpointer = getsp()
; ------------------------------------------
_getsp   proc  near
         mov   ax,sp
         ret
_getsp   endp
public   _keyhit
; ------- test for a keystroke, return 1 if key ready, 0 if not
;          k = keyhit();
;--------------------------------------
_keyhit proc   near
         mov   ah,1      ; BIOS read key status
         int   16h
         mov   ah,1      ; in case of ctrl-break
         jnz   keyret    ; if a key is ready
         int   28h       ; else, call int 28 to allow other TSRs in
         mov   ax,0      ; and return false
keyret: ret
_keyhit endp
public   _dores
; -------- swap the stack and call the popup function
_dores   proc  near
         cmp   _suspending,0
         jne   resret      ; don't pop up if the TSR is suspended
         mov   _running,1  ; prevent reentrant TSRs
         cli
         mov   intss,ss    ; save the interrupted stack
         mov   intsp,sp
         mov   ss,_myss    ; set the TSR's stack
         mov   sp,_stack
         sti
         call  _popup_tsr  ; pop up the TSR utility
         cli
         mov   ss,intss    ; restore the interrupted stack
         mov   sp,intsp
         sti
         mov   _running,0
resret: ret
_dores   endp
public   _newdisk
;--------- 0x13 (disk) BIOS Interrupt Service Routine
_newdisk proc  far
         push  ds
         push  bp
         mov   bp,dgroup   ; set ds to TSR's data segment
         mov   ds,bp
         inc   _diskflag   ; set disk in process flag
         pushf
         call  _olddisk    ; chain to old disk interrupt
         dec   _diskflag   ; reset disk in process flag
         pop   bp
         pop   ds
         ret   2           ; ret 2 passes on the flags
_newdisk endp
public   _newvideo
;--------- 0x10 (video) BIOS Interrupt Service Routine
_newvideo proc far
         push  ds
         push  bp
         mov   bp,dgroup   ; set ds to TSR's data segment
         mov   ds,bp
         inc   _videoflag  ; set video in process flag
         pop   bp          ; some EGA functions use bp in
         pushf
         call  _oldvideo   ; chain to old video interrupt
         dec   _videoflag  ; reset video in process flag
         pop   ds
         iret
_newvideo endp
public   _vpoke
; -------- insert a word into video memory
;        vpoke(unsigned vseg, unsigned adr, unsigned ch);
; ------------------------------------------
_vpoke   proc  near
         push  bp
         mov   bp,sp
         push  di
         push  es
         mov   cx,4[bp]  ; video board base address
         mov   es,cx
         mov   di,6[bp]  ; offset address from caller
         cmp   cx,45056  ;mono?
         jz    mono1
         mov   dx,986    ; video status port address
loop1:   in    al,dx     ; wait for retrace to quit
         test  al,1
         jnz   loop1
loop2:   in    al,dx     ; wait for retrace to start
         test  al,1
         jz    loop2
mono1:   mov   ax,8[bp]  ; word to insert
         stosw           ; insert it
         pop   es
         pop   di
         pop   bp
         ret
_vpoke   endp
public _vpeek
; -------- retrieve a word from video memory
;        unsigned vpeek(unsigned vseg, unsigned adr);
; ------------------------------------------
_vpeek   proc  near
         push  bp
         mov   bp,sp
         push  si
         push  ds
         mov   si,6[bp]  ; offset address
         mov   cx,4[bp]  ; video board base address
         mov   ds,cx
         cmp   cx,45056  ;mono?
         jz    mono2
         mov   dx,986    ; video status port address
loop3:   in    al,dx     ; wait for retrace to stop
         test  al,1
         jnz   loop3
loop4:   in    al,dx     ; wait for retrace to start
         test  al,1
         jz    loop4
mono2:   lodsw           ; get the word
         pop   ds
         pop   si
         pop   bp
         ret
_vpeek   endp
end
